"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.DEVNET_SERUM3_MARKETS = void 0;
const anchor_1 = require("@coral-xyz/anchor");
const web3_js_1 = require("@solana/web3.js");
const chai_1 = require("chai");
const fs_1 = __importDefault(require("fs"));
const mangoAccount_1 = require("../src/accounts/mangoAccount");
const perp_1 = require("../src/accounts/perp");
const client_1 = require("../src/client");
const constants_1 = require("../src/constants");
const utils_1 = require("../src/utils");
//
// An example for users based on high level api i.e. the client
// Create
// process.env.USER_KEYPAIR - mango account owner keypair path
// process.env.ADMIN_KEYPAIR - group admin keypair path (useful for automatically finding the group)
//
// This script deposits some tokens, places some serum orders, cancels them, places some perp orders
//
const DEVNET_MINTS = new Map([
    ['USDC', '8FRFC6MoGGkMFQwngccyu69VnYbzykGeez7ignHVAFSN'],
    ['BTC', '3UNBZ6o52WTWwjac2kPUb4FyodhU1vFkRJheu1Sh2TvU'],
    ['SOL', 'So11111111111111111111111111111111111111112'],
    ['ORCA', 'orcarKHSqC5CDDsGbho8GKvwExejWHxTqGzXgcewB9L'],
    ['MNGO', 'Bb9bsTQa1bGEtQ5KagGkvSHyuLqDWumFUcRqFusFNJWC'],
]);
exports.DEVNET_SERUM3_MARKETS = new Map([
    ['BTC/USDC', new web3_js_1.PublicKey('DW83EpHFywBxCHmyARxwj3nzxJd7MUdSeznmrdzZKNZB')],
    ['SOL/USDC', new web3_js_1.PublicKey('5xWpt56U1NCuHoAEtpLeUrQcxDkEpNfScjfLFaRzLPgR')],
]);
const GROUP_NUM = Number(process.env.GROUP_NUM || 0);
async function main() {
    const options = anchor_1.AnchorProvider.defaultOptions();
    const connection = new web3_js_1.Connection('https://mango.devnet.rpcpool.com', options);
    const user = web3_js_1.Keypair.fromSecretKey(Buffer.from(JSON.parse(fs_1.default.readFileSync(process.env.USER_KEYPAIR, 'utf-8'))));
    const userWallet = new anchor_1.Wallet(user);
    const userProvider = new anchor_1.AnchorProvider(connection, userWallet, options);
    const client = await client_1.MangoClient.connect(userProvider, 'devnet', constants_1.MANGO_V4_ID['devnet'], {
        idsSource: 'get-program-accounts',
    });
    console.log(`User ${userWallet.publicKey.toBase58()}`);
    // fetch group
    const admin = web3_js_1.Keypair.fromSecretKey(Buffer.from(JSON.parse(fs_1.default.readFileSync(process.env.ADMIN_KEYPAIR, 'utf-8'))));
    const group = await client.getGroupForCreator(admin.publicKey, GROUP_NUM);
    // create + fetch account
    console.log(`Creating mangoaccount...`);
    let mangoAccount = (await client.getOrCreateMangoAccount(group));
    await mangoAccount.reload(client);
    if (!mangoAccount) {
        throw new Error(`MangoAccount not found for user ${user.publicKey}`);
    }
    console.log(`...created/found mangoAccount ${mangoAccount.publicKey}`);
    // set delegate, and change name
    if (true) {
        console.log(`...changing mango account name, and setting a delegate`);
        const newName = 'my_changed_name';
        const randomKey = new web3_js_1.PublicKey('4ZkS7ZZkxfsC3GtvvsHP3DFcUeByU9zzZELS4r8HCELo');
        await client.editMangoAccount(group, mangoAccount, newName, randomKey);
        await mangoAccount.reload(client);
        (0, chai_1.expect)(mangoAccount.name).deep.equals(newName);
        (0, chai_1.expect)(mangoAccount.delegate).deep.equals(randomKey);
        const oldName = 'my_mango_account';
        console.log(`...resetting mango account name, and re-setting a delegate`);
        await client.editMangoAccount(group, mangoAccount, oldName, web3_js_1.PublicKey.default);
        await mangoAccount.reload(client);
        (0, chai_1.expect)(mangoAccount.name).deep.equals(oldName);
        (0, chai_1.expect)(mangoAccount.delegate).deep.equals(web3_js_1.PublicKey.default);
    }
    // expand account
    if (mangoAccount.tokens.length < 16 ||
        mangoAccount.serum3.length < 8 ||
        mangoAccount.perps.length < 8 ||
        mangoAccount.perpOpenOrders.length < 8) {
        console.log(`...expanding mango account to max 16 token positions, 8 serum3, 8 perp position and 8 perp oo slots, previous (tokens ${mangoAccount.tokens.length}, serum3 ${mangoAccount.serum3.length}, perps ${mangoAccount.perps.length}, perps oo ${mangoAccount.perpOpenOrders.length})`);
        let sig = await client.expandMangoAccount(group, mangoAccount, 16, 8, 8, 8);
        console.log(`sig https://explorer.solana.com/tx/${sig}?cluster=devnet`);
        await mangoAccount.reload(client);
        (0, chai_1.expect)(mangoAccount.tokens.length).equals(16);
        (0, chai_1.expect)(mangoAccount.serum3.length).equals(8);
        (0, chai_1.expect)(mangoAccount.perps.length).equals(8);
        (0, chai_1.expect)(mangoAccount.perpOpenOrders.length).equals(8);
    }
    // deposit and withdraw
    if (true) {
        console.log(`...depositing 50 USDC, 1 SOL, 1 MNGO`);
        // deposit USDC
        let oldBalance = mangoAccount.getTokenBalance(group.getFirstBankByMint(new web3_js_1.PublicKey(DEVNET_MINTS.get('USDC'))));
        await client.tokenDeposit(group, mangoAccount, new web3_js_1.PublicKey(DEVNET_MINTS.get('USDC')), 50);
        await mangoAccount.reload(client);
        let newBalance = mangoAccount.getTokenBalance(group.getFirstBankByMint(new web3_js_1.PublicKey(DEVNET_MINTS.get('USDC'))));
        (0, chai_1.expect)((0, utils_1.toUiDecimalsForQuote)(newBalance.sub(oldBalance)).toString()).equals('50');
        // deposit SOL
        await client.tokenDeposit(group, mangoAccount, new web3_js_1.PublicKey(DEVNET_MINTS.get('SOL')), 1);
        await mangoAccount.reload(client);
        // deposit MNGO
        await client.tokenDeposit(group, mangoAccount, new web3_js_1.PublicKey(DEVNET_MINTS.get('MNGO')), 1);
        await mangoAccount.reload(client);
        // withdraw USDC
        console.log(`...withdrawing 1 USDC`);
        oldBalance = mangoAccount.getTokenBalance(group.getFirstBankByMint(new web3_js_1.PublicKey(DEVNET_MINTS.get('USDC'))));
        await client.tokenWithdraw(group, mangoAccount, new web3_js_1.PublicKey(DEVNET_MINTS.get('USDC')), 1, true);
        await mangoAccount.reload(client);
        newBalance = mangoAccount.getTokenBalance(group.getFirstBankByMint(new web3_js_1.PublicKey(DEVNET_MINTS.get('USDC'))));
        (0, chai_1.expect)((0, utils_1.toUiDecimalsForQuote)(oldBalance.sub(newBalance)).toString()).equals('1');
        console.log(`...depositing 0.0005 BTC`);
        await client.tokenDeposit(group, mangoAccount, new web3_js_1.PublicKey(DEVNET_MINTS.get('BTC')), 0.0005);
        await mangoAccount.reload(client);
    }
    // Note: Disable for now until we have openbook devnet markets
    // if (true) {
    //   // serum3
    //   const asks = await group.loadSerum3AsksForMarket(
    //     client,
    //     DEVNET_SERUM3_MARKETS.get('BTC/USDC')!,
    //   );
    //   const lowestAsk = Array.from(asks!)[0];
    //   const bids = await group.loadSerum3BidsForMarket(
    //     client,
    //     DEVNET_SERUM3_MARKETS.get('BTC/USDC')!,
    //   );
    //   const highestBid = Array.from(bids!)![0];
    //   console.log(`...cancelling all existing serum3 orders`);
    //   if (
    //     Array.from(mangoAccount.serum3OosMapByMarketIndex.values()).length > 0
    //   ) {
    //     await client.serum3CancelAllOrders(
    //       group,
    //       mangoAccount,
    //       DEVNET_SERUM3_MARKETS.get('BTC/USDC')!,
    //       10,
    //     );
    //   }
    //   let price = 20;
    //   let qty = 0.0001;
    //   console.log(
    //     `...placing serum3 bid which would not be settled since its relatively low then midprice at ${price} for ${qty}`,
    //   );
    //   await client.serum3PlaceOrder(
    //     group,
    //     mangoAccount,
    //     DEVNET_SERUM3_MARKETS.get('BTC/USDC')!,
    //     Serum3Side.bid,
    //     price,
    //     qty,
    //     Serum3SelfTradeBehavior.decrementTake,
    //     Serum3OrderType.limit,
    //     Date.now(),
    //     10,
    //   );
    //   await mangoAccount.reload(client);
    //   let orders = await mangoAccount.loadSerum3OpenOrdersForMarket(
    //     client,
    //     group,
    //     DEVNET_SERUM3_MARKETS.get('BTC/USDC')!,
    //   );
    //   expect(orders[0].price).equals(20);
    //   expect(orders[0].size).equals(qty);
    //   price = lowestAsk.price + lowestAsk.price / 2;
    //   qty = 0.0001;
    //   console.log(
    //     `...placing serum3 bid way above midprice at ${price} for ${qty}`,
    //   );
    //   await client.serum3PlaceOrder(
    //     group,
    //     mangoAccount,
    //     DEVNET_SERUM3_MARKETS.get('BTC/USDC')!,
    //     Serum3Side.bid,
    //     price,
    //     qty,
    //     Serum3SelfTradeBehavior.decrementTake,
    //     Serum3OrderType.limit,
    //     Date.now(),
    //     10,
    //   );
    //   await mangoAccount.reload(client);
    //   price = highestBid.price - highestBid.price / 2;
    //   qty = 0.0001;
    //   console.log(
    //     `...placing serum3 ask way below midprice at ${price} for ${qty}`,
    //   );
    //   await client.serum3PlaceOrder(
    //     group,
    //     mangoAccount,
    //     DEVNET_SERUM3_MARKETS.get('BTC/USDC')!,
    //     Serum3Side.ask,
    //     price,
    //     qty,
    //     Serum3SelfTradeBehavior.decrementTake,
    //     Serum3OrderType.limit,
    //     Date.now(),
    //     10,
    //   );
    //   console.log(`...current own orders on OB`);
    //   orders = await mangoAccount.loadSerum3OpenOrdersForMarket(
    //     client,
    //     group,
    //     DEVNET_SERUM3_MARKETS.get('BTC/USDC')!,
    //   );
    //   for (const order of orders) {
    //     console.log(
    //       `  - order orderId ${order.orderId}, ${order.side}, ${order.price}, ${order.size}`,
    //     );
    //     console.log(`  - cancelling order with ${order.orderId}`);
    //     await client.serum3CancelOrder(
    //       group,
    //       mangoAccount,
    //       DEVNET_SERUM3_MARKETS.get('BTC/USDC')!,
    //       order.side === 'buy' ? Serum3Side.bid : Serum3Side.ask,
    //       order.orderId,
    //     );
    //   }
    //   console.log(`...current own orders on OB`);
    //   orders = await mangoAccount.loadSerum3OpenOrdersForMarket(
    //     client,
    //     group,
    //     DEVNET_SERUM3_MARKETS.get('BTC/USDC')!,
    //   );
    //   for (const order of orders) {
    //     console.log(order);
    //   }
    //   console.log(`...settling funds`);
    //   await client.serum3SettleFunds(
    //     group,
    //     mangoAccount,
    //     DEVNET_SERUM3_MARKETS.get('BTC/USDC')!,
    //   );
    // }
    if (true) {
        await mangoAccount.reload(client);
        console.log('...mangoAccount.getEquity() ' +
            (0, utils_1.toUiDecimalsForQuote)(mangoAccount.getEquity(group).toNumber()));
        console.log('...mangoAccount.getCollateralValue() ' +
            (0, utils_1.toUiDecimalsForQuote)(mangoAccount.getCollateralValue(group).toNumber()));
        console.log('...mangoAccount.getAssetsVal() ' +
            (0, utils_1.toUiDecimalsForQuote)(mangoAccount.getAssetsValue(group, mangoAccount_1.HealthType.init).toNumber()));
        console.log('...mangoAccount.getLiabsVal() ' +
            (0, utils_1.toUiDecimalsForQuote)(mangoAccount.getLiabsValue(group, mangoAccount_1.HealthType.init).toNumber()));
        console.log('...mangoAccount.getMaxWithdrawWithBorrowForToken(group, "SOL") ' +
            (0, utils_1.toUiDecimalsForQuote)(mangoAccount
                .getMaxWithdrawWithBorrowForToken(group, new web3_js_1.PublicKey(DEVNET_MINTS.get('SOL')))
                .toNumber()));
    }
    if (true) {
        function getMaxSourceForTokenSwapWrapper(src, tgt) {
            console.log(`getMaxSourceForTokenSwap ${src.padEnd(4)} ${tgt.padEnd(4)} ` +
                mangoAccount.getMaxSourceUiForTokenSwap(group, group.banksMapByName.get(src)[0].mint, group.banksMapByName.get(tgt)[0].mint, 1));
        }
        for (const srcToken of Array.from(group.banksMapByName.keys())) {
            for (const tgtToken of Array.from(group.banksMapByName.keys())) {
                getMaxSourceForTokenSwapWrapper(srcToken, tgtToken);
            }
        }
        const maxQuoteForSerum3BidUi = mangoAccount.getMaxQuoteForSerum3BidUi(group, exports.DEVNET_SERUM3_MARKETS.get('BTC/USDC'));
        console.log("...mangoAccount.getMaxQuoteForSerum3BidUi(group, 'BTC/USDC') " +
            maxQuoteForSerum3BidUi);
        const maxBaseForSerum3AskUi = mangoAccount.getMaxBaseForSerum3AskUi(group, exports.DEVNET_SERUM3_MARKETS.get('BTC/USDC'));
        console.log("...mangoAccount.getMaxBaseForSerum3AskUi(group, 'BTC/USDC') " +
            maxBaseForSerum3AskUi);
        console.log(`simHealthRatioWithSerum3BidUiChanges ${mangoAccount.simHealthRatioWithSerum3BidUiChanges(group, 785, exports.DEVNET_SERUM3_MARKETS.get('BTC/USDC'))}`);
        console.log(`simHealthRatioWithSerum3AskUiChanges ${mangoAccount.simHealthRatioWithSerum3AskUiChanges(group, 0.033, exports.DEVNET_SERUM3_MARKETS.get('BTC/USDC'))}`);
    }
    // perps
    if (true) {
        let sig;
        let perpMarket = group.getPerpMarketByName('BTC-PERP');
        const orders = await mangoAccount.loadPerpOpenOrdersForMarket(client, group, perpMarket.perpMarketIndex);
        for (const order of orders) {
            console.log(`Current order - ${order.uiPrice} ${order.uiSize} ${order.side}`);
        }
        console.log(`...cancelling all perp orders`);
        sig = await client.perpCancelAllOrders(group, mangoAccount, perpMarket.perpMarketIndex, 10);
        console.log(`sig https://explorer.solana.com/tx/${sig}?cluster=devnet`);
        // oracle pegged
        try {
            const clientId = Math.floor(Math.random() * 99999);
            const price = group.banksMapByName.get('BTC')[0].uiPrice;
            console.log(`...placing perp pegged bid ${clientId} at oracle price ${perpMarket.uiPrice}`);
            const sig = await client.perpPlaceOrderPegged(group, mangoAccount, perpMarket.perpMarketIndex, perp_1.PerpOrderSide.bid, -5, perpMarket.uiPrice + 5, 0.01, price * 0.011, clientId, perp_1.PerpOrderType.limit, false, 0, 1);
            console.log(`sig https://explorer.solana.com/tx/${sig}?cluster=devnet`);
        }
        catch (error) {
            console.log(error);
        }
        try {
            const clientId = Math.floor(Math.random() * 99999);
            const price = group.banksMapByName.get('BTC')[0].uiPrice;
            console.log(`...placing perp pegged bid ${clientId} at oracle price ${perpMarket.uiPrice}`);
            const sig = await client.perpPlaceOrderPegged(group, mangoAccount, perpMarket.perpMarketIndex, perp_1.PerpOrderSide.ask, 5, perpMarket.uiPrice - 5, 0.01, price * 0.011, clientId, perp_1.PerpOrderType.limit, false, 0, 1);
            console.log(`sig https://explorer.solana.com/tx/${sig}?cluster=devnet`);
        }
        catch (error) {
            console.log(error);
        }
        await logBidsAndAsks(client, group);
        sig = await client.perpCancelAllOrders(group, mangoAccount, perpMarket.perpMarketIndex, 10);
        console.log(`sig https://explorer.solana.com/tx/${sig}?cluster=devnet`);
        // scenario 1
        // bid max perp
        try {
            const clientId = Math.floor(Math.random() * 99999);
            await mangoAccount.reload(client);
            await group.reloadAll(client);
            const price = group.banksMapByName.get('BTC')[0].uiPrice -
                Math.floor(Math.random() * 100);
            const quoteQty = mangoAccount.getMaxQuoteForPerpBidUi(group, perpMarket.perpMarketIndex);
            const baseQty = quoteQty / price;
            console.log(` simHealthRatioWithPerpBidUiChanges - ${mangoAccount.simHealthRatioWithPerpBidUiChanges(group, perpMarket.perpMarketIndex, baseQty)}`);
            console.log(`...placing max qty perp bid  clientId ${clientId} at price ${price}, base ${baseQty}, quote ${quoteQty}`);
            const sig = await client.perpPlaceOrder(group, mangoAccount, perpMarket.perpMarketIndex, perp_1.PerpOrderSide.bid, price, baseQty, quoteQty, clientId, perp_1.PerpOrderType.limit, false, 0, //Date.now() + 200,
            1);
            console.log(`sig https://explorer.solana.com/tx/${sig}?cluster=devnet`);
        }
        catch (error) {
            console.log(error);
        }
        console.log(`...cancelling all perp orders`);
        sig = await client.perpCancelAllOrders(group, mangoAccount, perpMarket.perpMarketIndex, 10);
        console.log(`sig https://explorer.solana.com/tx/${sig}?cluster=devnet`);
        // bid max perp + some
        try {
            const clientId = Math.floor(Math.random() * 99999);
            const price = group.banksMapByName.get('BTC')[0].uiPrice -
                Math.floor(Math.random() * 100);
            const quoteQty = mangoAccount.getMaxQuoteForPerpBidUi(group, perpMarket.perpMarketIndex) * 1.02;
            const baseQty = quoteQty / price;
            console.log(`...placing max qty * 1.02 perp bid clientId ${clientId} at price ${price}, base ${baseQty}, quote ${quoteQty}`);
            const sig = await client.perpPlaceOrder(group, mangoAccount, perpMarket.perpMarketIndex, perp_1.PerpOrderSide.bid, price, baseQty, quoteQty, clientId, perp_1.PerpOrderType.limit, false, 0, //Date.now() + 200,
            1);
            console.log(`sig https://explorer.solana.com/tx/${sig}?cluster=devnet`);
        }
        catch (error) {
            console.log(error);
            console.log('Errored out as expected');
        }
        // bid max ask
        try {
            const clientId = Math.floor(Math.random() * 99999);
            const price = group.banksMapByName.get('BTC')[0].uiPrice +
                Math.floor(Math.random() * 100);
            const baseQty = mangoAccount.getMaxBaseForPerpAskUi(group, perpMarket.perpMarketIndex);
            console.log(` simHealthRatioWithPerpAskUiChanges - ${mangoAccount.simHealthRatioWithPerpAskUiChanges(group, perpMarket.perpMarketIndex, baseQty)}`);
            const quoteQty = baseQty * price;
            console.log(`...placing max qty perp ask clientId ${clientId} at price ${price}, base ${baseQty}, quote ${quoteQty}`);
            const sig = await client.perpPlaceOrder(group, mangoAccount, perpMarket.perpMarketIndex, perp_1.PerpOrderSide.ask, price, baseQty, quoteQty, clientId, perp_1.PerpOrderType.limit, false, 0, //Date.now() + 200,
            1);
            console.log(`sig https://explorer.solana.com/tx/${sig}?cluster=devnet`);
        }
        catch (error) {
            console.log(error);
        }
        // bid max ask + some
        try {
            const clientId = Math.floor(Math.random() * 99999);
            const price = group.banksMapByName.get('BTC')[0].uiPrice +
                Math.floor(Math.random() * 100);
            const baseQty = mangoAccount.getMaxBaseForPerpAskUi(group, perpMarket.perpMarketIndex) *
                1.02;
            const quoteQty = baseQty * price;
            console.log(`...placing max qty perp ask * 1.02 clientId ${clientId} at price ${price}, base ${baseQty}, quote ${quoteQty}`);
            const sig = await client.perpPlaceOrder(group, mangoAccount, perpMarket.perpMarketIndex, perp_1.PerpOrderSide.ask, price, baseQty, quoteQty, clientId, perp_1.PerpOrderType.limit, false, 0, //Date.now() + 200,
            1);
            console.log(`sig https://explorer.solana.com/tx/${sig}?cluster=devnet`);
        }
        catch (error) {
            console.log(error);
            console.log('Errored out as expected');
        }
        console.log(`...cancelling all perp orders`);
        sig = await client.perpCancelAllOrders(group, mangoAccount, perpMarket.perpMarketIndex, 10);
        console.log(`sig https://explorer.solana.com/tx/${sig}?cluster=devnet`);
        // scenario 2
        // make + take orders
        try {
            const clientId = Math.floor(Math.random() * 99999);
            const price = group.banksMapByName.get('BTC')[0].uiPrice;
            console.log(`...placing perp bid ${clientId} at ${price}`);
            const sig = await client.perpPlaceOrder(group, mangoAccount, perpMarket.perpMarketIndex, perp_1.PerpOrderSide.bid, price, 0.01, price * 0.01, clientId, perp_1.PerpOrderType.limit, false, 0, //Date.now() + 200,
            1);
            console.log(`sig https://explorer.solana.com/tx/${sig}?cluster=devnet`);
        }
        catch (error) {
            console.log(error);
        }
        try {
            const clientId = Math.floor(Math.random() * 99999);
            const price = group.banksMapByName.get('BTC')[0].uiPrice;
            console.log(`...placing perp ask ${clientId} at ${price}`);
            const sig = await client.perpPlaceOrder(group, mangoAccount, perpMarket.perpMarketIndex, perp_1.PerpOrderSide.ask, price, 0.01, price * 0.011, clientId, perp_1.PerpOrderType.limit, false, 0, //Date.now() + 200,
            1);
            console.log(`sig https://explorer.solana.com/tx/${sig}?cluster=devnet`);
        }
        catch (error) {
            console.log(error);
        }
        // // should be able to cancel them : know bug
        // console.log(`...cancelling all perp orders`);
        // sig = await client.perpCancelAllOrders(group, mangoAccount, perpMarket.perpMarketIndex, 10);
        // console.log(`sig https://explorer.solana.com/tx/${sig}?cluster=devnet`);
        await perpMarket?.loadEventQueue(client);
        const fr = perpMarket?.getCurrentFundingRate(await perpMarket.loadBids(client), await perpMarket.loadAsks(client));
        console.log(`current funding rate per hour is ${fr}`);
        const eq = await perpMarket?.loadEventQueue(client);
        console.log(`raw events - ${JSON.stringify(eq.eventsSince(new anchor_1.BN(0)), null, 2)}`);
        // sleep so that keeper can catch up
        await new Promise((r) => setTimeout(r, 2000));
        // make+take orders should have cancelled each other, and if keeper has already cranked, then should not appear in position or we see a small quotePositionNative
        await group.reloadAll(client);
        await mangoAccount.reload(client);
        console.log(`${mangoAccount.toString(group)}`);
    }
    process.exit();
}
async function logBidsAndAsks(client, group) {
    await group.reloadAll(client);
    const perpMarket = group.getPerpMarketByName('BTC-PERP');
    const res = [
        (await perpMarket?.loadBids(client)).items(),
        (await perpMarket?.loadAsks(client)).items(),
    ];
    console.log(`bids ${JSON.stringify(Array.from(res[0]), null, 2)}`);
    console.log(`asks ${JSON.stringify(Array.from(res[1]), null, 2)}`);
    return res;
}
main();
